# Kompira: シンプルで軽量な次世代運用自動化プラットフォーム

## 1. Kompiraとは

Kompira は IT 運用管理業務の自動化を支援するための基盤システムです。
ジョブフローを記述することで、様々な運用業務、管理業務、さらには障害の復旧処理まで自動化することができます。

## 2. 動作環境

Kompira 1.6 の動作環境は以下の通りです。

### 2.1. 対応システム

* アーキテクチャ: x86_64
* オペレーティングシステム:
    * Red Hat Enterprise Linux (RHEL): 8.x 系（推奨）または 7.6 以上（代替）
    * CentOS:  7.6 以上（代替）
    * CentOS Stream: 8（代替）
    * Amazon Linux 2
* Pythonバージョン: Python 3.6
* ネットワークインタフェース: 1つ以上

※CentOS 8 系は EOL に伴いサポート外となりました（インストールに失敗します）。CentOS Stream 8 のご利用をご検討ください。

### 2.2. ハードウェア要件

* CPU：2GHz以上
* メモリ：8GB以上
* HDD：250GB以上

※ 必要なスペックは Kompira 上で動作するジョブフロー規模、自動化の処理要件によって異なります。都度ご相談ください。
※ 記載要件は最低レベルで記載しております。お客様の運用環境によっては異なる場合がございますのでご了承ください。

### 2.3. 利用するポート

Kompira が利用するポート番号は以下の通りです。

#### 2.3.1. 外部から内部への方向

* Kompira GUI, REST APIへのアクセス： http(80), https(443)
* kompira_sendevt によるイベント送信： amqp(5672)

#### 2.3.2. 内部から外部への方向

* Kompira からメール受信： POP(110), IMAP(143), POPS(995), IMAPS(993)
* Kompira からメール送信： SMTP(25), SMTP-TLS(587), SMTPS(465)
* Kompira から外部機器へのアクセス： ssh(22), telnet(23), wsman-http(5985), wsman-https(5986)
* Kompira から外部 API の呼び出し： http(80), https(443)

#### 2.3.3. 冗長構成の場合の ACT/SBY 間通信

* rabbitmq(4369, 5672, 25672)
* postgresql(5432)
* pcsd(2224)
* corosync(UDP/5404, UDP/5405)

## 3. インストール

### 3.1. インストール手順

パッケージを展開したら root 権限で ./install.sh を実行してください。

```
# ./install.sh
```

* インストール先のサーバは、OS が新規インストール状態であることを推奨します。
* Kompira のインストールには、インターネット接続環境が必要です。
* RedHat にインストールする場合は、事前にサブスクリプションを登録しておく必要があります。
* CentOS/RHEL 7.6 でインストールに失敗する場合、Python 3.6 を事前にインストールしておき、install.sh のオプションに `--skip-python3-install` を指定して下さい。


インストールに成功したサーバにブラウザでアクセスするとログイン画面が表示されますので、以下のアカウントでログインすることで Kompira をはじめることができます。

* ユーザ名: `root`
* パスワード: `root`

ログイン後の画面の右上にある「ヘルプ」をクリックすると、オンラインマニュアルが表示されますので使い方の参考にしてください。

### 3.2. proxy 環境下でのインストール

以下のように --proxy オプションを付けて install.sh を実行してください。

```
# ./install.sh --proxy proxy:3128
```

* オプションに渡す "proxy" や "3128" の部分は導入環境のプロキシサーバのホスト名（またはIPアドレス）やポート番号に合わせてください。

認証付きプロキシサーバの場合は、以下のように "user" にユーザー名を、"password" にパスワードを指定して install.sh を実行してください。 

```
# ./install.sh --proxy user:password@proxy:3128
```

### 3.3. インストール時の言語設定

デフォルトではインストール先の Linux サーバのシステムの言語設定に合わせてインストールされます。(ただし、対応する言語は日本語と英語のみで、エンコーディング指定は UTF-8 のみサポートしています)

システムの言語設定と異なる言語でインストールしたい場合は、--locale-lang オプションを指定してインストールしてください。たとえば、英語モードでインストールする場合は、以下のように指定します。

```
# ./install.sh --locale-lang en_US.UTF-8
```

## 4. 注意点
### 4.1. 利用上の注意点
- v1.6.4 から添付ファイルがデータベースに保存されるようになり、また合わせてサイズ制限が緩和されました。データベース上では添付ファイルの実サイズより大きい容量を必要とするため、大きな添付ファイルを扱う場合はディスクの空き容量にご注意ください。
- v1.6.4 からサーバ上に添付ファイルの実体は保持しなくなりますので、サーバ上に添付ファイルの実体が存在することを前提としたジョブフローなどは正常に動作しなくなります。

### 4.2. アップデート時の注意点
- v1.6.3 以前から v1.6.4 以降にアップデートした場合、既存の添付ファイルは kompirad 起動時に自動的にデータベースに保存されます。そのため、添付ファイルの数が多い場合は起動に時間がかかる場合があります。また、データベース保存に移行した後も、添付ファイルに対応するサーバ上の実ファイルはそのまま残っていますので、不要な場合は手動で `/var/opt/kompira/upload` を削除してください。

### 4.3. 冗長構成での注意点
- v.1.6.6 以降で冗長構成時のアップデート手順が変更され、install.sh の後の pcs cluster start が不要になりました。詳しくは管理ガイドの「1.9.3.アップデート」をご参照ください。
- CentOS 7 環境における冗長構成アップデートについて
    - 対象：v1.6.0～v1.6.2.post1 から v1.6.2.post2 以降へのアップデート
    - 詳細については以下をご確認ください。
        - https://docbase.io/posts/2118544/sharing/aaab9c34-93a2-4af6-85e0-7b50cef07d3a
- 冗長構成を v1.6.3 以前から v1.6.4 以降にアップデートした場合、pacemaker 配下で lsyncd サービスが動作する設定のままとなっています。v1.6.4 環境で lsyncd が動作していても影響はありませんが、lsyncd サービスを停止させる場合は冗長構成のアップデートが正常に終えたあとに、root 権限で以下のコマンドを実行してください。

    ```
    # pcs resource delete res_lsyncd
    ```

## 5. Kompira ライセンス

[Kompira Enterprise ライセンス利用規約](https://www.kompira.jp/Kompira_terms.pdf)に同意の上、ダウンロードページからダウンロードして下さい。

Kompira の使用には、ライセンス登録が必要です。詳しくは [license@kompira.jp](mailto:license@kompira.jp) までご連絡ください。

※ ご利用の Kompira のバージョンに依らず、最新のライセンス利用規約が適用されます。

## 6. Kompira 関連の情報

### 6.1. Kompira ニュース

Kompira の最新情報については [Kompiraニュース](https://bitbucket.org/kompira/package/wiki/News) をご参照ください。

### 6.2. Kompira 運用自動化コラム

Kompira の実践的な使い方やジョブフローの書き方については [運用自動化コラム](https://www.kompira.jp/column/) を参考にしてみてください。

### 6.3. Kompira コミュニティサイト

Kompira の使い方が分からない場合などは、 [コミュニティ> KompiraEnterprise関連](https://kompira.zendesk.com/hc/ja/community/topics/360000014321-KompiraEnterprise%E9%96%A2%E9%80%A3) を参考にしてみてください。同じような質問や回答が見つからない場合は、新たに投稿してみてください。